// Straightforward implementation
#include <iostream>
#include <algorithm>
#include <queue>
#include <vector>
#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#define ll long long
#define INFLL 1000000000000000000LL
#define mp make_pair

using namespace std;

const int N = 2111111;
const int M = 2111111;

class DSU {
 private:
  int p[N];
 public:
  DSU() {
    memset(p, -1, sizeof(p));
  }
  int who(int x) {
    if (p[x] == -1) return x;
    return p[x] = who(p[x]);
  }
  void un(int x, int y) {
    x = who(x);
    y = who(y);
    if (rand() & 1) p[x] = y;
    else            p[y] = x;
  }
};

class Graph {
  private:
  	int e;
  	void _add(int a, int b, int weight) {
  		nxt[e] = last[a];
  		v[e] = b;
  		w[e] = weight;
  		last[a] = e;
  		e++;
  	}
  public:
  	int nxt[M], v[M], w[M];
  	int last[N], n, m;
  	void clear() {
		  memset(last, -1, sizeof(last));
  		e = 0;
  	}
  	Graph() {
  		this->clear();
  	}
  	Graph(int n, int m) : n(n), m(m) {
  		this->clear();
  	}
  	void add(int u, int v, int w) {
  		_add(u, v, w);
  		_add(v, u, w);
  	}
  	void read() {
  		this->clear();
  		scanf("%d%d", &n, &m);
  		for (int i = 0; i < m; i++) {
  			int a, b, weight;
  			scanf("%d%d%d", &a, &b, &weight);
  			a--; b--;
  			add(a, b, weight);
  		}
  	}
  	ll mst();
  	ll dijkstra();
};

DSU dsu;

ll Graph::mst() {
  ll ans = 0;
  vector<pair<int, pair<int, int> > > edges;
  for (int i = 0; i < n; i++)
    for (int j = last[i]; j >= 0; j = nxt[j])
      if (i > v[j])
        edges.push_back(mp(w[j], mp(i, v[j])));
  sort(edges.begin(), edges.end());
  assert(edges.size() == m);
  for (int i = 0; i < m; i++) {
    int u = edges[i].second.first;
    int v = edges[i].second.second;
    if (dsu.who(u) != dsu.who(v)) {
      dsu.un(u, v);
      ans += edges[i].first;
    }
  }
  return ans;
}

ll Graph::dijkstra() {
  priority_queue<pair<ll, int>, vector<pair<ll, int> >, greater<pair<ll, int> > > Q;
  vector<int> used(n, false);
  Q.push(mp(0LL, 0));
  while (!Q.empty()) {
    pair<ll, int> cur = Q.top(); Q.pop();
    if (cur.second == n - 1) return cur.first;
    if (used[cur.second]) continue;
    used[cur.second] = true;
    for (int i = last[cur.second]; i >= 0; i = nxt[i]) {
      if (!used[v[i]]) {
        Q.push(mp(cur.first + w[i], v[i]));
      }
    }
  }
  assert(0);
}

void mul(const Graph &a, const Graph &b, Graph &c) {
	c.n = a.n * b.n;
  c.m = a.m * b.n + a.n * b.m;
	for (int i = 0; i < a.n; i++) {
		for (int j = 0; j < b.n; j++) {
			for (int k = b.last[j]; k >= 0; k = b.nxt[k]) {
        if (j > b.v[k]) continue;
				int x = i * b.n + j;
				int y = i * b.n + b.v[k];
		    if (x < y) {
          c.add(x, y, b.w[k]);
        }
			}
			for (int k = a.last[i]; k >= 0; k = a.nxt[k]) {
				int x = i * b.n + j;
				int y = a.v[k] * b.n + j;
				if (x < y) {
          c.add(x, y, a.w[k]);
        }
			}
		}
	}
}

Graph a, b, c;

int main() {
  #ifndef SHTRIX
  freopen("cartesius.dat", "rt", stdin);
  freopen("cartesius.sol", "wt", stdout);
  #endif
  int query;
  scanf("%d", &query);
	a.read();
	b.read();
	mul(a, b, c);
	if (query == 1) {
		cout << c.dijkstra() << endl;
	} else if (query == 2) {
		cout << c.mst() << endl;
	} else {
    assert(0);
  }
}